==> JAOPCA-1.16.5-3.4.6.44-sources.jar: thelm/jaopca/compat/kubejs/utils/MaterialForm.java <==
package thelm.jaopca.compat.kubejs.utils;

import com.google.common.collect.TreeBasedTable;

import dev.latvian.kubejs.fluid.EmptyFluidStackJS;
import dev.latvian.kubejs.fluid.FluidStackJS;
import dev.latvian.kubejs.item.ItemStackJS;
import net.minecraft.block.Block;
import net.minecraft.block.BlockState;
import thelm.jaopca.api.blocks.IBlockProvider;
import thelm.jaopca.api.fluids.IFluidProvider;
import thelm.jaopca.api.forms.IForm;
import thelm.jaopca.api.materialforms.IMaterialFormInfo;
import thelm.jaopca.api.materials.IMaterial;
import thelm.jaopca.utils.MiscHelper;

public class MaterialForm {

	private static final TreeBasedTable<IForm, IMaterial, MaterialForm> MATERIAL_FORM_WRAPPERS = TreeBasedTable.create();
	private final IMaterialFormInfo info;

	public static MaterialForm getMaterialFormWrapper(IForm form, IMaterial material) {
		MaterialForm materialForm = MATERIAL_FORM_WRAPPERS.get(form, material);
		if(materialForm == null) {
			IMaterialFormInfo info = form.getType().getMaterialFormInfo(form, material);
			materialForm = new MaterialForm(info);
		}
		return materialForm;
	}

	private MaterialForm(IMaterialFormInfo info) {
		this.info = info;
	}

	public IMaterialFormInfo getInternal() {
		return info;
	}

	public Form getForm() {
		return Form.getFormWrapper(info.getMaterialForm().getForm());
	}

	public Material getMaterial() {
		return Material.getMaterialWrapper(info.getMaterialForm().getMaterial());
	}

	public String asTag() {
		return MiscHelper.INSTANCE.getTagLocation(
				info.getMaterialForm().getForm().getSecondaryName(), info.getMaterialForm().getMaterial().getName(),
				info.getMaterialForm().getForm().getTagSeparator()).
				toString();
	}

	public ItemStackJS asItemStack(int count) {
		return ItemStackJS.of(info);
	}

	public ItemStackJS asItemStack() {
		return asItemStack(1);
	}

	public FluidStackJS asFluidStack(int amount) {
		if(info instanceof IFluidProvider) {
			return FluidStackJS.of(((IFluidProvider)info).asFluid(), amount, null);
		}
		return EmptyFluidStackJS.INSTANCE;
	}

	public Block asBlock() {
		if(info instanceof IBlockProvider) {
			return ((IBlockProvider)info).asBlock();
		}
		return null;
	}

	public BlockState asBlockState() {
		if(info instanceof IBlockProvider) {
			return (((IBlockProvider)info).asBlock().func_176223_P());
		}
		return null;
	}
}

==> JAOPCA-1.18.2-4.1.21.52-sources.jar: thelm/jaopca/compat/kubejs/utils/MaterialForm.java <==
package thelm.jaopca.compat.kubejs.utils;

import com.google.common.collect.TreeBasedTable;

import dev.latvian.mods.kubejs.fluid.EmptyFluidStackJS;
import dev.latvian.mods.kubejs.fluid.FluidStackJS;
import dev.latvian.mods.kubejs.item.ItemStackJS;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import thelm.jaopca.api.blocks.IBlockLike;
import thelm.jaopca.api.fluids.IFluidLike;
import thelm.jaopca.api.forms.IForm;
import thelm.jaopca.api.materialforms.IMaterialFormInfo;
import thelm.jaopca.api.materials.IMaterial;
import thelm.jaopca.utils.MiscHelper;

public class MaterialForm {

	private static final TreeBasedTable<IForm, IMaterial, MaterialForm> MATERIAL_FORM_WRAPPERS = TreeBasedTable.create();
	private final IMaterialFormInfo info;

	public static MaterialForm getMaterialFormWrapper(IForm form, IMaterial material) {
		MaterialForm materialForm = MATERIAL_FORM_WRAPPERS.get(form, material);
		if(materialForm == null) {
			IMaterialFormInfo info = form.getType().getMaterialFormInfo(form, material);
			materialForm = new MaterialForm(info);
		}
		return materialForm;
	}

	private MaterialForm(IMaterialFormInfo info) {
		this.info = info;
	}

	public IMaterialFormInfo getInternal() {
		return info;
	}

	public Form getForm() {
		return Form.getFormWrapper(info.getMaterialForm().getForm());
	}

	public Material getMaterial() {
		return Material.getMaterialWrapper(info.getMaterialForm().getMaterial());
	}

	public String asTag() {
		return MiscHelper.INSTANCE.getTagLocation(
				info.getMaterialForm().getForm().getSecondaryName(), info.getMaterialForm().getMaterial().getName(),
				info.getMaterialForm().getForm().getTagSeparator()).
				toString();
	}

	public ItemStackJS asItemStack(int count) {
		return ItemStackJS.of(info);
	}

	public ItemStackJS asItemStack() {
		return asItemStack(1);
	}

	public FluidStackJS asFluidStack(int amount) {
		if(info instanceof IFluidLike fluid) {
			return FluidStackJS.of(fluid.asFluid(), amount, null);
		}
		return EmptyFluidStackJS.INSTANCE;
	}

	public Block asBlock() {
		if(info instanceof IBlockLike block) {
			return block.asBlock();
		}
		return null;
	}

	public BlockState asBlockState() {
		if(info instanceof IBlockLike block) {
			return block.asBlock().defaultBlockState();
		}
		return null;
	}
}
